@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014  Free Software Foundation, Inc.
+ * Copyright (C) 2014-2017  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -17,8 +17,6 @@
  *	Paulo Cesar Pereira de Andrade
  */
 
-#define rc(value)			jit_class_##value
-#define rn(reg)				(jit_regno(_rvs[jit_regno(reg)].spec))
 #define jit_arg_reg_p(i)		((i) >= 0 && (i) < 6)
 #define jit_arg_f_reg_p(i)		((i) >= 0 && (i) < 6)
 #if __BYTE_ORDER == __LITTLE_ENDIAN
@@ -33,6 +31,30 @@
 #  define F_DISP			8 - sizeof(jit_float32_t)
 #endif
 
+/*
+ * Types
+ */
+/*
+ * What I could understand from gcc/config/alpha/alpha.c:alpha_build_builtin_va_list()
+ * and other helpers, as well as objdump of simple test programs; could not
+ * get gdb working on the test system I had access...
+ *
+ * base-48 to base is where up to 6 float registers are saved.
+ * base to base+48 is where up to 6 integer registers are saved.
+ * base+48... is where varargs arguments are stored.
+ *
+ *	if (offset < 48) {
+ *		if (type == double)
+ *			offset -= 48;
+ *	}
+ *	load(reg, base, offset);
+ *	offset += 8;
+ */
+typedef struct jit_va_list {
+    jit_pointer_t	base;
+    jit_word_t		offset;
+} jit_va_list_t;
+
 /*
  * Prototypes
  */
@@ -155,6 +177,10 @@ _jit_prolog(jit_state_t *_jit)
     jit_alloc((jit_pointer_t *)&_jitc->function->regoff,
 	      _jitc->reglen * sizeof(jit_int32_t));
 
+    /* _no_link here does not mean the jit_link() call can be removed
+     * by rewriting as:
+     * _jitc->function->prolog = jit_new_node(jit_code_prolog);
+     */
     _jitc->function->prolog = jit_new_node_no_link(jit_code_prolog);
     jit_link(_jitc->function->prolog);
     _jitc->function->prolog->w.w = offset;
@@ -179,70 +205,107 @@ _jit_allocai(jit_state_t *_jit, jit_int32_t length)
 	default:	_jitc->function->self.aoff &= -8;	break;
     }
     _jitc->function->self.aoff -= length;
+    if (!_jitc->realize) {
+	jit_inc_synth_ww(allocai, _jitc->function->self.aoff, length);
+	jit_dec_synth();
+    }
     return (_jitc->function->self.aoff);
 }
 
+void
+_jit_allocar(jit_state_t *_jit, jit_int32_t u, jit_int32_t v)
+{
+    jit_int32_t		 reg;
+    assert(_jitc->function != NULL);
+    jit_inc_synth_ww(allocar, u, v);
+    if (!_jitc->function->allocar) {
+	_jitc->function->aoffoff = jit_allocai(sizeof(jit_int32_t));
+	_jitc->function->allocar = 1;
+    }
+    reg = jit_get_reg(jit_class_gpr);
+    jit_negr(reg, v);
+    jit_andi(reg, reg, -8);
+    jit_ldxi_i(u, JIT_FP, _jitc->function->aoffoff);
+    jit_addr(u, u, reg);
+    jit_addr(JIT_SP, JIT_SP, reg);
+    jit_stxi_i(_jitc->function->aoffoff, JIT_FP, u);
+    jit_unget_reg(reg);
+    jit_dec_synth();
+}
+
 void
 _jit_ret(jit_state_t *_jit)
 {
     jit_node_t		*instr;
-
     assert(_jitc->function != NULL);
-
+    jit_inc_synth(ret);
     /* jump to epilog */
     instr = jit_jmpi();
     jit_patch_at(instr, _jitc->function->epilog);
+    jit_dec_synth();
 }
 
 void
 _jit_retr(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr, u);
     if (JIT_RET != u)
 	jit_movr(JIT_RET, u);
     else
 	jit_live(JIT_RET);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti(jit_state_t *_jit, jit_word_t u)
 {
+    jit_inc_synth_w(reti, u);
     jit_movi(JIT_RET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_retr_f(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr_f, u);
     if (u != JIT_FRET)
 	jit_movr_f(JIT_FRET, u);
     else
 	jit_live(JIT_FRET);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti_f(jit_state_t *_jit, jit_float32_t u)
 {
+    jit_inc_synth_f(reti_f, u);
     jit_movi_f(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_retr_d(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr_d, u);
     if (u != JIT_FRET)
 	jit_movr_d(JIT_FRET, u);
     else
 	jit_live(JIT_FRET);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti_d(jit_state_t *_jit, jit_float64_t u)
 {
+    jit_inc_synth_d(reti_d, u);
     jit_movi_d(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
@@ -263,10 +326,32 @@ _jit_arg_register_p(jit_state_t *_jit, jit_node_t *u)
     return (jit_arg_f_reg_p(u->u.w));
 }
 
+void
+_jit_ellipsis(jit_state_t *_jit)
+{
+    jit_inc_synth(jit_code_ellipsis);
+    if (_jitc->prepare) {
+	jit_link_prepare();
+	assert(!(_jitc->function->call.call & jit_call_varargs));
+	_jitc->function->call.call |= jit_call_varargs;
+    }
+    else {
+	jit_link_prolog();
+	assert(!(_jitc->function->self.call & jit_call_varargs));
+	_jitc->function->self.call |= jit_call_varargs;
+
+	/* Allocate va_list like object in the stack */
+	_jitc->function->vaoff = jit_allocai(sizeof(jit_va_list_t));
+	_jitc->function->vagp = _jitc->function->self.argi;
+    }
+    jit_dec_synth();
+}
+
 jit_node_t *
 _jit_arg(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function != NULL);
     if (jit_arg_reg_p(_jitc->function->self.argi))
 	offset = _jitc->function->self.argi++;
@@ -274,13 +359,17 @@ _jit_arg(jit_state_t *_jit)
 	offset = _jitc->function->self.size;
 	_jitc->function->self.size += 8;
     }
-    return (jit_new_node_w(jit_code_arg, offset));
+    node = jit_new_node_ww(jit_code_arg, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 jit_node_t *
 _jit_arg_f(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function != NULL);
     if (jit_arg_f_reg_p(_jitc->function->self.argi))
 	offset = _jitc->function->self.argi++;
@@ -288,13 +377,17 @@ _jit_arg_f(jit_state_t *_jit)
 	offset = _jitc->function->self.size;
 	_jitc->function->self.size += 8;
     }
-    return (jit_new_node_w(jit_code_arg_f, offset));
+    node = jit_new_node_ww(jit_code_arg_f, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 jit_node_t *
 _jit_arg_d(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function != NULL);
     if (jit_arg_f_reg_p(_jitc->function->self.argi))
 	offset = _jitc->function->self.argi++;
@@ -302,87 +395,106 @@ _jit_arg_d(jit_state_t *_jit)
 	offset = _jitc->function->self.size;
 	_jitc->function->self.size += 8;
     }
-    return (jit_new_node_w(jit_code_arg_d, offset));
+    node = jit_new_node_ww(jit_code_arg_d, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 void
 _jit_getarg_c(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_c, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_c(u, _A0 - v->u.w);
     else
 	jit_ldxi_c(u, _FP, v->u.w + C_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_uc(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_uc, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_uc(u, _A0 - v->u.w);
     else
 	jit_ldxi_uc(u, _FP, v->u.w + C_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_s(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_s, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_s(u, _A0 - v->u.w);
     else
 	jit_ldxi_s(u, _FP, v->u.w + S_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_us(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_us, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_us(u, _A0 - v->u.w);
     else
 	jit_ldxi_us(u, _FP, v->u.w + S_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_i(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_i, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_i(u, _A0 - v->u.w);
     else
 	jit_ldxi_i(u, _FP, v->u.w + I_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_ui(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_ui, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_ui(u, _A0 - v->u.w);
     else
 	jit_ldxi_ui(u, _FP, v->u.w + I_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_l(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_l, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movr(u, _A0 - v->u.w);
     else
 	jit_ldxi_l(u, _FP, v->u.w);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(putargr, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movr(_A0 - v->u.w, u);
     else
 	jit_stxi(v->u.w, _FP, u);
+    jit_dec_synth();
 }
 
 void
@@ -390,6 +502,7 @@ _jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(putargi, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movi(_A0 - v->u.w, u);
     else {
@@ -398,26 +511,31 @@ _jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v)
 	jit_stxi(v->u.w, _FP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_f(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_wp(getarg_f, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_f(u, _F16 - v->u.w);
     else
 	jit_ldxi_f(u, _FP, v->u.w + F_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr_f(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_wp(putargr_f, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_f(_F16 - v->u.w, u);
     else
 	jit_stxi_f(v->u.w, _FP, u + F_DISP);
+    jit_dec_synth();
 }
 
 void
@@ -425,6 +543,7 @@ _jit_putargi_f(jit_state_t *_jit, jit_float32_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_fp(putargi_f, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movi_f(_F16 - v->u.w, u);
     else {
@@ -433,26 +552,31 @@ _jit_putargi_f(jit_state_t *_jit, jit_float32_t u, jit_node_t *v)
 	jit_stxi_f(v->u.w, _FP, regno + F_DISP);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_d(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_wp(getarg_d, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_d(u, _F16 - v->u.w);
     else
 	jit_ldxi_d(u, _FP, v->u.w);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr_d(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_wp(putargr_d, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_d(_F16 - v->u.w, u);
     else
 	jit_stxi_d(v->u.w, _FP, u);
+    jit_dec_synth();
 }
 
 void
@@ -460,6 +584,7 @@ _jit_putargi_d(jit_state_t *_jit, jit_float64_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_dp(putargi_d, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movi_d(_F16 - v->u.w, u);
     else {
@@ -468,12 +593,15 @@ _jit_putargi_d(jit_state_t *_jit, jit_float64_t u, jit_node_t *v)
 	jit_stxi_d(v->u.w, _FP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function != NULL);
+    jit_inc_synth_w(pushargr, u);
+    jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movr(_A0 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -482,6 +610,7 @@ _jit_pushargr(jit_state_t *_jit, jit_int32_t u)
 	jit_stxi(_jitc->function->call.size, JIT_SP, u);
 	_jitc->function->call.size += 8;
     }
+    jit_dec_synth();
 }
 
 void
@@ -489,6 +618,8 @@ _jit_pushargi(jit_state_t *_jit, jit_int64_t u)
 {
     jit_int32_t		regno;
     assert(_jitc->function != NULL);
+    jit_inc_synth_w(pushargi, u);
+    jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movi(_A0 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -500,12 +631,15 @@ _jit_pushargi(jit_state_t *_jit, jit_int64_t u)
 	_jitc->function->call.size += 8;
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function != NULL);
+    jit_inc_synth_w(pushargr_f, u);
+    jit_link_prepare();
     if (jit_arg_f_reg_p(_jitc->function->call.argi)) {
 	jit_movr_f(_F16 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -514,6 +648,7 @@ _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
 	jit_stxi_f(_jitc->function->call.size + F_DISP, JIT_SP, u);
 	_jitc->function->call.size += 8;
     }
+    jit_dec_synth();
 }
 
 void
@@ -521,6 +656,8 @@ _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
 {
     jit_int32_t		regno;
     assert(_jitc->function != NULL);
+    jit_inc_synth_f(pushargi_f, u);
+    jit_link_prepare();
     if (jit_arg_f_reg_p(_jitc->function->call.argi)) {
 	jit_movi_f(_F16 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -532,12 +669,15 @@ _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
 	_jitc->function->call.size += 8;
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function != NULL);
+    jit_inc_synth_w(pushargr_d, u);
+    jit_link_prepare();
     if (jit_arg_f_reg_p(_jitc->function->call.argi)) {
 	jit_movr_d(_F16 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -546,6 +686,7 @@ _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
 	jit_stxi_d(_jitc->function->call.size, JIT_SP, u);
 	_jitc->function->call.size += 8;
     }
+    jit_dec_synth();
 }
 
 void
@@ -553,6 +694,8 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
 {
     jit_int32_t		regno;
     assert(_jitc->function != NULL);
+    jit_inc_synth_d(pushargi_d, u);
+    jit_link_prepare();
     if (jit_arg_f_reg_p(_jitc->function->call.argi)) {
 	jit_movi_d(_F16 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -564,6 +707,7 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
 	_jitc->function->call.size += 8;
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 jit_bool_t
@@ -592,86 +736,106 @@ void
 _jit_finishr(jit_state_t *_jit, jit_int32_t r0)
 {
     jit_node_t		*call;
-
     assert(_jitc->function != NULL);
+    jit_inc_synth_w(finishr, r0);
     if (_jitc->function->self.alen < _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
     call = jit_callr(r0);
     call->v.w = call->w.w = _jitc->function->self.argi;
     _jitc->function->call.argi = _jitc->function->call.size = 0;
     _jitc->prepare = 0;
+    jit_dec_synth();
 }
 
 jit_node_t *
 _jit_finishi(jit_state_t *_jit, jit_pointer_t i0)
 {
     jit_node_t		*call;
-
     assert(_jitc->function != NULL);
+    jit_inc_synth_w(finishi, (jit_word_t)i0);
     if (_jitc->function->self.alen < _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
     call = jit_calli(i0);
     call->v.w = call->w.w = _jitc->function->self.argf;
     _jitc->function->call.argi = _jitc->function->call.size = 0;
     _jitc->prepare = 0;
+    jit_dec_synth();
     return (call);
 }
 
 void
 _jit_retval_c(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_c, r0);
     jit_extr_c(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_uc(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_uc, r0);
     jit_extr_uc(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_s(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_s, r0);
     jit_extr_s(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_us(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_us, r0);
     jit_extr_us(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_i(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_i, r0);
     jit_extr_i(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_ui(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_ui, r0);
     jit_extr_ui(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_l(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_l, r0);
     if (r0 != JIT_RET)
 	jit_movr(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_f(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_f, r0);
     if (r0 != JIT_FRET)
 	jit_movr_f(r0, JIT_FRET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_d(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_d, r0);
     if (r0 != JIT_FRET)
 	jit_movr_d(r0, JIT_FRET);
+    jit_dec_synth();
 }
 
 jit_pointer_t
@@ -686,9 +850,15 @@ _emit_code(jit_state_t *_jit)
 	jit_node_t	*node;
 	jit_uint8_t	*data;
 	jit_word_t	 word;
+#if DEVEL_DISASSEMBLER
+	jit_word_t	 prevw;
+#endif
 	jit_int32_t	 const_offset;
 	jit_int32_t	 patch_offset;
     } undo;
+#if DEVEL_DISASSEMBLER
+    jit_word_t		 prevw;
+#endif
 
     _jitc->function = NULL;
 
@@ -780,12 +950,16 @@ _emit_code(jit_state_t *_jit)
 		    patch(word, node);					\
 		}							\
 		break
+#if DEVEL_DISASSEMBLER
+    prevw = _jit->pc.w;
+#endif
     for (node = _jitc->head; node; node = node->next) {
 	if (_jit->pc.uc >= _jitc->code.end)
 	    return (NULL);
 
 #if DEVEL_DISASSEMBLER
-	node->offset = _jit->pc.w;
+	node->offset = (jit_uword_t)_jit->pc.w - (jit_uword_t)prevw;
+	prevw = _jit->pc.w;
 #endif
 	value = jit_classify(node->code);
 	jit_regarg_set(node, value);
@@ -1191,6 +1365,9 @@ _emit_code(jit_state_t *_jit)
 		_jitc->function = _jitc->functions.ptr + node->w.w;
 		undo.node = node;
 		undo.word = _jit->pc.w;
+#if DEVEL_DISASSEMBLER
+		undo.prevw = prevw;
+#endif
 		undo.patch_offset = _jitc->patches.offset;
 	    restart_function:
 		_jitc->again = 0;
@@ -1208,6 +1385,9 @@ _emit_code(jit_state_t *_jit)
 		    temp->flag &= ~jit_flag_patch;
 		    node = undo.node;
 		    _jit->pc.w = undo.word;
+#if DEVEL_DISASSEMBLER
+		    prevw = undo.prevw;
+#endif
 		    _jitc->patches.offset = undo.patch_offset;
 		    goto restart_function;
 		}
@@ -1217,9 +1397,42 @@ _emit_code(jit_state_t *_jit)
 		epilog(node);
 		_jitc->function = NULL;
 		break;
-	    case jit_code_live:
+	    case jit_code_va_start:
+		vastart(rn(node->u.w));
+		break;
+	    case jit_code_va_arg:
+		vaarg(rn(node->u.w), rn(node->v.w));
+		break;
+	    case jit_code_va_arg_d:
+		vaarg_d(rn(node->u.w), rn(node->v.w));
+		break;
+	    case jit_code_live:			case jit_code_ellipsis:
+	    case jit_code_allocai:		case jit_code_allocar:
 	    case jit_code_arg:
 	    case jit_code_arg_f:		case jit_code_arg_d:
+	    case jit_code_va_end:
+	    case jit_code_ret:
+	    case jit_code_retr:			case jit_code_reti:
+	    case jit_code_retr_f:		case jit_code_reti_f:
+	    case jit_code_retr_d:		case jit_code_reti_d:
+	    case jit_code_getarg_c:		case jit_code_getarg_uc:
+	    case jit_code_getarg_s:		case jit_code_getarg_us:
+	    case jit_code_getarg_i:		case jit_code_getarg_ui:
+	    case jit_code_getarg_l:
+	    case jit_code_getarg_f:		case jit_code_getarg_d:
+	    case jit_code_putargr:		case jit_code_putargi:
+	    case jit_code_putargr_f:		case jit_code_putargi_f:
+	    case jit_code_putargr_d:		case jit_code_putargi_d:
+	    case jit_code_pushargr:		case jit_code_pushargi:
+	    case jit_code_pushargr_f:		case jit_code_pushargi_f:
+	    case jit_code_pushargr_d:		case jit_code_pushargi_d:
+	    case jit_code_retval_c:		case jit_code_retval_uc:
+	    case jit_code_retval_s:		case jit_code_retval_us:
+	    case jit_code_retval_i:
+	    case jit_code_retval_ui:		case jit_code_retval_l:
+	    case jit_code_retval_f:		case jit_code_retval_d:
+	    case jit_code_prepare:
+	    case jit_code_finishr:		case jit_code_finishi:
 		break;
 	    default:
 		abort();
@@ -1239,7 +1452,8 @@ _emit_code(jit_state_t *_jit)
 	    }
 	}
 	jit_regarg_clr(node, value);
-	assert(_jitc->regarg == jit_carry == _NOREG ? 0 : (1 << jit_carry));
+	assert(_jitc->regarg == (jit_carry == _NOREG) ? 0 : (1 << jit_carry));
+	assert(_jitc->synth == 0);
 	/* update register live state */
 	jit_reglive(node);
     }
