@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012, 2013  Free Software Foundation, Inc.
+ * Copyright (C) 2012-2017  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -804,6 +804,8 @@ static void _vfp_stxr_d(jit_state_t*,jit_int32_t,jit_int32_t,jit_int32_t);
 static void _vfp_stxi_f(jit_state_t*,jit_word_t,jit_int32_t,jit_int32_t);
 #  define vfp_stxi_d(i0,r0,r1)		_vfp_stxi_d(_jit,i0,r0,r1)
 static void _vfp_stxi_d(jit_state_t*,jit_word_t,jit_int32_t,jit_int32_t);
+#  define vfp_vaarg_d(r0, r1)		_vfp_vaarg_d(_jit, r0, r1)
+static void _vfp_vaarg_d(jit_state_t*, jit_int32_t, jit_int32_t);
 #endif
 
 #if CODE
@@ -2301,6 +2303,26 @@ _vfp_stxi_d(jit_state_t *_jit, jit_word_t i0, jit_int32_t r0, jit_int32_t r1)
 	jit_unget_reg(reg);
     }
 }
+
+static void
+_vfp_vaarg_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1)
+{
+    jit_int32_t		reg;
+
+    assert(_jitc->function->self.call & jit_call_varargs);
+
+    /* Adjust pointer. */
+    reg = jit_get_reg(jit_class_gpr);
+    andi(rn(reg), r1, 7);
+    addr(r1, r1, rn(reg));
+    jit_unget_reg(reg);
+
+    /* Load argument. */
+    vfp_ldr_d(r0, r1);
+
+    /* Update stack pointer. */
+    addi(r1, r1, sizeof(jit_float64_t));
+}
 #  undef dbopi
 #  undef fbopi
 #  undef dopi
