@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013  Free Software Foundation, Inc.
+ * Copyright (C) 2013-2017  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -20,8 +20,6 @@
 #include <lightning.h>
 #include <lightning/jit_private.h>
 
-#define rc(value)			jit_class_##value
-#define rn(reg)				(jit_regno(_rvs[jit_regno(reg)].spec))
 #define jit_arg_reg_p(i)		(i >= 0 && i < 4)
 
 #define PROTO				1
@@ -29,6 +27,11 @@
 #  include "jit_hppa-fpu.c"
 #undef PROTO
 
+/*
+ * Types
+ */
+typedef jit_pointer_t	jit_va_list;
+
 /*
  * Prototypes
  */
@@ -160,6 +163,10 @@ _jit_prolog(jit_state_t *_jit)
     jit_alloc((jit_pointer_t *)&_jitc->function->regoff,
 	      _jitc->reglen * sizeof(jit_int32_t));
 
+    /* _no_link here does not mean the jit_link() call can be removed
+     * by rewriting as:
+     * _jitc->function->prolog = jit_new_node(jit_code_prolog);
+     */
     _jitc->function->prolog = jit_new_node_no_link(jit_code_prolog);
     jit_link(_jitc->function->prolog);
     _jitc->function->prolog->w.w = offset;
@@ -191,63 +198,99 @@ _jit_allocai(jit_state_t *_jit, jit_int32_t length)
 	    _jitc->function->self.aoff = (_jitc->function->self.aoff + 7) & -8;
 	    break;
     }
+    if (!_jitc->realize) {
+	jit_inc_synth_ww(allocai, _jitc->function->self.aoff, length);
+	jit_dec_synth();
+    }
     offset = _jitc->function->self.aoff;
     _jitc->function->self.aoff += length;
     return (offset);
 }
 
+void
+_jit_allocar(jit_state_t *_jit, jit_int32_t u, jit_int32_t v)
+{
+    jit_int32_t		 reg;
+    assert(_jitc->function);
+    jit_inc_synth_ww(allocar, u, v);
+    if (!_jitc->function->allocar) {
+	_jitc->function->aoffoff = jit_allocai(sizeof(jit_int32_t));
+	_jitc->function->allocar = 1;
+    }
+    reg = jit_get_reg(jit_class_gpr);
+    jit_addi(reg, v, 63);
+    jit_andi(reg, reg, -64);
+    jit_ldxi_i(u, JIT_FP, _jitc->function->aoffoff);
+    jit_addr(JIT_SP, JIT_SP, reg);
+    jit_stxi_i(_jitc->function->aoffoff, JIT_FP, u);
+    jit_unget_reg(reg);
+    jit_dec_synth();
+}
+
 void
 _jit_ret(jit_state_t *_jit)
 {
     jit_node_t		*instr;
-
     assert(_jitc->function);
-
+    jit_inc_synth(ret);
     /* jump to epilog */
     instr = jit_jmpi();
     jit_patch_at(instr, _jitc->function->epilog);
+    jit_dec_synth();
 }
 
 void
 _jit_retr(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr, u);
     jit_movr(JIT_RET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti(jit_state_t *_jit, jit_word_t u)
 {
+    jit_inc_synth_w(reti, u);
     jit_movi(JIT_RET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_retr_f(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr_f, u);
     jit_movr_f(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti_f(jit_state_t *_jit, jit_float32_t u)
 {
+    jit_inc_synth_f(reti_f, u);
     jit_movi_f(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_retr_d(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr_d, u);
     jit_movr_d(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti_d(jit_state_t *_jit, jit_float64_t u)
 {
+    jit_inc_synth_d(reti_d, u);
     jit_movi_d(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
@@ -267,36 +310,64 @@ _jit_arg_register_p(jit_state_t *_jit, jit_node_t *u)
     return (jit_arg_reg_p(u->u.w));
 }
 
+void
+_jit_ellipsis(jit_state_t *_jit)
+{
+    jit_inc_synth(ellipsis);
+    if (_jitc->prepare) {
+	jit_link_prepare();
+	assert(!(_jitc->function->call.call & jit_call_varargs));
+	_jitc->function->call.call |= jit_call_varargs;
+    }
+    else {
+	jit_link_prolog();
+	assert(!(_jitc->function->self.call & jit_call_varargs));
+	_jitc->function->self.call |= jit_call_varargs;
+
+	_jitc->function->vagp = _jitc->function->self.argi;
+    }
+    jit_dec_synth();
+}
+
 jit_node_t *
 _jit_arg(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function);
     _jitc->function->self.size -= sizeof(jit_word_t);
     if (jit_arg_reg_p(_jitc->function->self.argi))
 	offset = _jitc->function->self.argi++;
     else
 	offset = _jitc->function->self.size;
-    return (jit_new_node_w(jit_code_arg, offset));
+    node = jit_new_node_ww(jit_code_arg, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 jit_node_t *
 _jit_arg_f(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function);
     _jitc->function->self.size -= sizeof(jit_word_t);
     if (jit_arg_reg_p(_jitc->function->self.argi))
 	offset = _jitc->function->self.argi++;
     else
 	offset = _jitc->function->self.size;
-    return (jit_new_node_w(jit_code_arg_f, offset));
+    node = jit_new_node_ww(jit_code_arg_f, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 jit_node_t *
 _jit_arg_d(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function);
     if (_jitc->function->self.argi & 1) {
 	++_jitc->function->self.argi;
@@ -313,67 +384,82 @@ _jit_arg_d(jit_state_t *_jit)
 	    _jitc->function->self.size -= sizeof(jit_word_t);
 	offset = _jitc->function->self.size;
     }
-    return (jit_new_node_w(jit_code_arg_d, offset));
+    node = jit_new_node_ww(jit_code_arg_d, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 void
 _jit_getarg_c(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_c, u, v);
     if (v->u.w >= 0)
 	jit_extr_c(u, _R26 - v->u.w);
     else
 	jit_ldxi_c(u, JIT_FP, v->u.w + 3);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_uc(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_uc, u, v);
     if (v->u.w >= 0)
 	jit_extr_uc(u, _R26 - v->u.w);
     else
 	jit_ldxi_uc(u, JIT_FP, v->u.w + 3);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_s(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_s, u, v);
     if (v->u.w >= 0)
 	jit_extr_s(u, _R26 - v->u.w);
     else
 	jit_ldxi_s(u, JIT_FP, v->u.w + 2);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_us(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_us, u, v);
     if (v->u.w >= 0)
 	jit_extr_us(u, _R26 - v->u.w);
     else
 	jit_ldxi_us(u, JIT_FP, v->u.w + 2);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_i(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_i, u, v);
     if (v->u.w >= 0)
 	jit_movr(u, _R26 - v->u.w);
     else
 	jit_ldxi_i(u, JIT_FP, v->u.w);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(putargr, u, v);
     if (v->u.w >= 0)
 	jit_movr(_R26 - v->u.w, u);
     else
 	jit_stxi(v->u.w, JIT_FP, u);
+    jit_dec_synth();
 }
 
 void
@@ -381,6 +467,7 @@ _jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(putargi, u, v);
     if (v->u.w >= 0)
 	jit_movi(_R26 - v->u.w, u);
     else {
@@ -389,26 +476,31 @@ _jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v)
 	jit_stxi(v->u.w, JIT_FP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_f(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_wp(getarg_f, u, v);
     if (v->u.w >= 0)
 	jit_movr_f(u, _F4 - v->u.w);
     else
 	jit_ldxi_f(u, JIT_FP, v->u.w);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr_f(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_wp(putargr_f, u, v);
     if (v->u.w >= 0)
 	jit_movr_f(_F4 - v->u.w, u);
     else
 	jit_stxi_f(v->u.w, JIT_FP, u);
+    jit_dec_synth();
 }
 
 void
@@ -416,6 +508,7 @@ _jit_putargi_f(jit_state_t *_jit, jit_float32_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_fp(putargi_f, u, v);
     if (v->u.w >= 0)
 	jit_movi_f(_R26 - v->u.w, u);
     else {
@@ -424,26 +517,31 @@ _jit_putargi_f(jit_state_t *_jit, jit_float32_t u, jit_node_t *v)
 	jit_stxi_f(v->u.w, JIT_FP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_d(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_wp(getarg_d, u, v);
     if (v->u.w >= 0)
 	jit_movr_d(u, _F4 - v->u.w);
     else
 	jit_ldxi_d(u, JIT_FP, v->u.w);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr_d(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_wp(putargr_d, u, v);
     if (v->u.w >= 0)
 	jit_movr_d(_F4 - v->u.w, u);
     else
 	jit_stxi_d(v->u.w, JIT_FP, u);
+    jit_dec_synth();
 }
 
 void
@@ -451,6 +549,7 @@ _jit_putargi_d(jit_state_t *_jit, jit_float64_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_dp(putargi_d, u, v);
     if (v->u.w >= 0)
 	jit_movi_d(_R26 - v->u.w, u);
     else {
@@ -459,12 +558,15 @@ _jit_putargi_d(jit_state_t *_jit, jit_float64_t u, jit_node_t *v)
 	jit_stxi_d(v->u.w, JIT_FP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function);
+    jit_inc_synth_w(pushargr, u);
+    jit_link_prepare();
     _jitc->function->call.size -= sizeof(jit_word_t);
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movr(_R26 - _jitc->function->call.argi, u);
@@ -472,6 +574,7 @@ _jit_pushargr(jit_state_t *_jit, jit_int32_t u)
     }
     else
 	jit_stxi(_jitc->function->call.size + params_offset, JIT_SP, u);
+    jit_dec_synth();
 }
 
 void
@@ -479,6 +582,8 @@ _jit_pushargi(jit_state_t *_jit, jit_word_t u)
 {
     jit_int32_t		 regno;
     assert(_jitc->function);
+    jit_inc_synth_w(pushargi, u);
+    jit_link_prepare();
     _jitc->function->call.size -= sizeof(jit_word_t);
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movi(_R26 - _jitc->function->call.argi, u);
@@ -490,16 +595,20 @@ _jit_pushargi(jit_state_t *_jit, jit_word_t u)
 	jit_stxi(_jitc->function->call.size + params_offset, JIT_SP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function);
+    jit_inc_synth_w(pushargr_f, u);
+    jit_link_prepare();
     _jitc->function->call.size -= sizeof(jit_word_t);
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movr_f(_F4 - _jitc->function->call.argi, u);
 #if !defined(__hpux)
+	/* HP-UX appears to always pass float arguments in gpr registers */
 	if (_jitc->function->call.call & jit_call_varargs)
 #endif
 	{
@@ -511,6 +620,7 @@ _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
     }
     else
 	jit_stxi_f(_jitc->function->call.size + params_offset, JIT_SP, u);
+    jit_dec_synth();
 }
 
 void
@@ -518,10 +628,13 @@ _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
 {
     jit_int32_t		 regno;
     assert(_jitc->function);
+    jit_inc_synth_f(pushargi_f, u);
+    jit_link_prepare();
     _jitc->function->call.size -= sizeof(jit_word_t);
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movi_f(_F4 - _jitc->function->call.argi, u);
 #if !defined(__hpux)
+	/* HP-UX appears to always pass float arguments in gpr registers */
 	if (_jitc->function->call.call & jit_call_varargs)
 #endif
 	{
@@ -538,12 +651,15 @@ _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
 	jit_stxi_f(_jitc->function->call.size + params_offset, JIT_SP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function);
+    jit_inc_synth_w(pushargr_d, u);
+    jit_link_prepare();
     _jitc->function->call.size -= sizeof(jit_float64_t);
     if (_jitc->function->call.argi & 1) {
 	++_jitc->function->call.argi;
@@ -552,6 +668,7 @@ _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movr_d(_F4 - (_jitc->function->call.argi + 1), u);
 #if !defined(__hpux)
+	/* HP-UX appears to always pass float arguments in gpr registers */
 	if (_jitc->function->call.call & jit_call_varargs)
 #endif
 	{
@@ -569,6 +686,7 @@ _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
 	    _jitc->function->call.size -= sizeof(jit_word_t);
 	jit_stxi_d(_jitc->function->call.size + params_offset, JIT_SP, u);
     }
+    jit_dec_synth();
 }
 
 void
@@ -576,6 +694,8 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
 {
     jit_int32_t		 regno;
     assert(_jitc->function);
+    jit_inc_synth_d(pushargi_d, u);
+    jit_link_prepare();
     _jitc->function->call.size -= sizeof(jit_float64_t);
     if (_jitc->function->call.argi & 1) {
 	++_jitc->function->call.argi;
@@ -584,6 +704,7 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movi_d(_F4 - (_jitc->function->call.argi + 1), u);
 #if !defined(__hpux)
+	/* HP-UX appears to always pass float arguments in gpr registers */
 	if (_jitc->function->call.call & jit_call_varargs)
 #endif
 	{
@@ -604,6 +725,7 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
 	jit_stxi_d(_jitc->function->call.size + params_offset, JIT_SP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 jit_bool_t
@@ -631,12 +753,14 @@ _jit_finishr(jit_state_t *_jit, jit_int32_t r0)
 {
     jit_node_t		*call;
     assert(_jitc->function);
+    jit_inc_synth_w(finishr, r0);
     if (_jitc->function->self.alen > _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
     call = jit_callr(r0);
     call->v.w = call->w.w = _jitc->function->call.argi;
     _jitc->function->call.argi = _jitc->function->call.size = 0;
     _jitc->prepare = 0;
+    jit_dec_synth();
 }
 
 jit_node_t *
@@ -644,55 +768,71 @@ _jit_finishi(jit_state_t *_jit, jit_pointer_t i0)
 {
     jit_node_t		*node;
     assert(_jitc->function);
+    jit_inc_synth_w(finishi, (jit_word_t)i0);
     if (_jitc->function->self.alen > _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
     node = jit_calli(i0);
     node->v.w = node->w.w = _jitc->function->call.argi;
     _jitc->function->call.argi = _jitc->function->call.size = 0;
     _jitc->prepare = 0;
+    jit_dec_synth();
     return (node);
 }
 
 void
 _jit_retval_c(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_c, r0);
     jit_extr_c(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_uc(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_uc, r0);
     jit_extr_uc(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_s(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_s, r0);
     jit_extr_s(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_us(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_us, r0);
     jit_extr_us(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_i(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_i, r0);
     jit_movr(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_f(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_f, r0);
     jit_movr_f(r0, JIT_FRET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_d(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_d, r0);
     jit_movr_d(r0, JIT_FRET);
+    jit_dec_synth();
 }
 
 jit_pointer_t
@@ -706,8 +846,14 @@ _emit_code(jit_state_t *_jit)
     struct {
 	jit_node_t	*node;
 	jit_word_t	 word;
+#if DEVEL_DISASSEMBLER
+	jit_word_t	 prevw;
+#endif
 	jit_int32_t	 patch_offset;
     } undo;
+#if DEVEL_DISASSEMBLER
+    jit_word_t		 prevw;
+#endif
 
     _jitc->function = NULL;
 
@@ -800,12 +946,16 @@ _emit_code(jit_state_t *_jit)
 		    patch(word, node);					\
 		}							\
 		break
+#if DEVEL_DISASSEMBLER
+    prevw = _jit->pc.w;
+#endif
     for (node = _jitc->head; node; node = node->next) {
 	if (_jit->pc.uc >= _jitc->code.end)
 	    return (NULL);
 
 #if DEVEL_DISASSEMBLER
-	node->offset = _jit->pc.w;
+	node->offset = (jit_uword_t)_jit->pc.w - (jit_uword_t)prevw;
+	prevw = _jit->pc.w;
 #endif
 	value = jit_classify(node->code);
 	jit_regarg_set(node, value);
@@ -1194,6 +1344,9 @@ _emit_code(jit_state_t *_jit)
 		_jitc->function = _jitc->functions.ptr + node->w.w;
 		undo.node = node;
 		undo.word = _jit->pc.w;
+#if DEVEL_DISASSEMBLER
+		undo.prevw = prevw;
+#endif
 		undo.patch_offset = _jitc->patches.offset;
 	    restart_function:
 		_jitc->again = 0;
@@ -1211,6 +1364,9 @@ _emit_code(jit_state_t *_jit)
 		    temp->flag &= ~jit_flag_patch;
 		    node = undo.node;
 		    _jit->pc.w = undo.word;
+#if DEVEL_DISASSEMBLER
+		    prevw = undo.prevw;
+#endif
 		    _jitc->patches.offset = undo.patch_offset;
 		    goto restart_function;
 		}
@@ -1220,15 +1376,46 @@ _emit_code(jit_state_t *_jit)
 		epilog(node);
 		_jitc->function = NULL;
 		break;
+	    case jit_code_va_start:
+		vastart(rn(node->u.w));
+		break;
+	    case jit_code_va_arg:
+		vaarg(rn(node->u.w), rn(node->v.w));
+		break;
+	    case jit_code_va_arg_d:
+		vaarg_d(rn(node->u.w), rn(node->v.w));
+		break;
 	    case jit_code_live:
-	    case jit_code_arg:
+	    case jit_code_arg:			case jit_code_ellipsis:
+	    case jit_code_allocai:		case jit_code_allocar:
 	    case jit_code_arg_f:		case jit_code_arg_d:
+	    case jit_code_va_end:
+	    case jit_code_ret:
+	    case jit_code_retr:			case jit_code_reti:
+	    case jit_code_retr_f:		case jit_code_reti_f:
+	    case jit_code_retr_d:		case jit_code_reti_d:
+	    case jit_code_getarg_c:		case jit_code_getarg_uc:
+	    case jit_code_getarg_s:		case jit_code_getarg_us:
+	    case jit_code_getarg_i:
+	    case jit_code_getarg_f:		case jit_code_getarg_d:
+	    case jit_code_putargr:		case jit_code_putargi:
+	    case jit_code_putargr_f:		case jit_code_putargi_f:
+	    case jit_code_putargr_d:		case jit_code_putargi_d:
+	    case jit_code_pushargr:		case jit_code_pushargi:
+	    case jit_code_pushargr_f:		case jit_code_pushargi_f:
+	    case jit_code_pushargr_d:		case jit_code_pushargi_d:
+	    case jit_code_retval_c:		case jit_code_retval_uc:
+	    case jit_code_retval_s:		case jit_code_retval_us:
+	    case jit_code_retval_i:
+	    case jit_code_retval_f:		case jit_code_retval_d:
+	    case jit_code_prepare:
+	    case jit_code_finishr:		case jit_code_finishi:
 		break;
 	    default:
 		abort();
 	}
 	jit_regarg_clr(node, value);
-	assert(_jitc->regarg == 0);
+	assert(_jitc->regarg == 0 && _jitc->synth == 0);
 	/* update register live state */
 	jit_reglive(node);
     }
