@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013  Free Software Foundation, Inc.
+ * Copyright (C) 2013-2017  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -20,8 +20,6 @@
 #include <lightning.h>
 #include <lightning/jit_private.h>
 
-#define rc(value)			jit_class_##value
-#define rn(reg)				(jit_regno(_rvs[jit_regno(reg)].spec))
 #define jit_arg_reg_p(i)		((i) >= 0 && (i) < 8)
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 #  define C_DISP			0
@@ -35,6 +33,11 @@
 #  define F_DISP			8 - sizeof(jit_float32_t)
 #endif
 
+/*
+ * Types
+ */
+typedef jit_pointer_t jit_va_list_t;
+
 /*
  * Prototypes
  */
@@ -272,6 +275,10 @@ _jit_prolog(jit_state_t *_jit)
     jit_alloc((jit_pointer_t *)&_jitc->function->regoff,
 	      _jitc->reglen * sizeof(jit_int32_t));
 
+    /* _no_link here does not mean the jit_link() call can be removed
+     * by rewriting as:
+     * _jitc->function->prolog = jit_new_node(jit_code_prolog);
+     */
     _jitc->function->prolog = jit_new_node_no_link(jit_code_prolog);
     jit_link(_jitc->function->prolog);
     _jitc->function->prolog->w.w = offset;
@@ -296,61 +303,98 @@ _jit_allocai(jit_state_t *_jit, jit_int32_t length)
 	default:	_jitc->function->self.aoff &= -8;	break;
     }
     _jitc->function->self.aoff -= length;
+    if (!_jitc->realize) {
+	jit_inc_synth_ww(allocai, _jitc->function->self.aoff, length);
+	jit_dec_synth();
+    }
     return (_jitc->function->self.aoff);
 }
 
+void
+_jit_allocar(jit_state_t *_jit, jit_int32_t u, jit_int32_t v)
+{
+    jit_int32_t		 reg;
+    assert(_jitc->function);
+    jit_inc_synth_ww(allocar, u, v);
+    if (!_jitc->function->allocar) {
+	_jitc->function->aoffoff = jit_allocai(sizeof(jit_int32_t));
+	_jitc->function->allocar = 1;
+    }
+    reg = jit_get_reg(jit_class_gpr);
+    jit_negr(reg, v);
+    jit_andi(reg, reg, -16);
+    jit_ldxi_i(u, JIT_FP, _jitc->function->aoffoff);
+    jit_addr(u, u, reg);
+    jit_addr(JIT_SP, JIT_SP, reg);
+    jit_stxi_i(_jitc->function->aoffoff, JIT_FP, u);
+    jit_unget_reg(reg);
+    jit_dec_synth();
+}
+
 void
 _jit_ret(jit_state_t *_jit)
 {
     jit_node_t		*instr;
-
     assert(_jitc->function);
-
+    jit_inc_synth(ret);
     /* jump to epilog */
     instr = jit_jmpi();
     jit_patch_at(instr, _jitc->function->epilog);
+    jit_dec_synth();
 }
 
 void
 _jit_retr(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr, u);
     jit_movr(JIT_RET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti(jit_state_t *_jit, jit_word_t u)
 {
+    jit_inc_synth_w(reti, u);
     jit_movi(JIT_RET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_retr_f(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr_f, u);
     jit_movr_f(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti_f(jit_state_t *_jit, jit_float32_t u)
 {
+    jit_inc_synth_f(reti_f, u);
     jit_movi_f(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_retr_d(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr_d, u);
     jit_movr_d(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti_d(jit_state_t *_jit, jit_float64_t u)
 {
+    jit_inc_synth_d(reti_d, u);
     jit_movi_d(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
@@ -370,11 +414,29 @@ _jit_arg_register_p(jit_state_t *_jit, jit_node_t *u)
     return (jit_arg_reg_p(u->u.w));
 }
 
+void
+_jit_ellipsis(jit_state_t *_jit)
+{
+    jit_inc_synth(ellipsis);
+    if (_jitc->prepare) {
+	assert(!(_jitc->function->call.call & jit_call_varargs));
+	_jitc->function->call.call |= jit_call_varargs;
+	jit_link_prepare();
+    }
+    else {
+	assert(!(_jitc->function->self.call & jit_call_varargs));
+	_jitc->function->self.call |= jit_call_varargs;
+	_jitc->function->vagp = _jitc->function->self.argi;
+	jit_link_prolog();
+    }
+    jit_dec_synth();
+}
+
 jit_node_t *
 _jit_arg(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
-
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function);
     if (jit_arg_reg_p(_jitc->function->self.argi))
 	offset = _jitc->function->self.argi++;
@@ -382,117 +444,152 @@ _jit_arg(jit_state_t *_jit)
 	offset = _jitc->function->self.size;
 	_jitc->function->self.size += sizeof(jit_word_t);
     }
-    return (jit_new_node_w(jit_code_arg, offset));
+    node = jit_new_node_ww(jit_code_arg, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 jit_node_t *
 _jit_arg_f(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
-
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function);
-    if (jit_arg_reg_p(_jitc->function->self.argi))
-	offset = _jitc->function->self.argi++;
-    else {
+    if (jit_arg_reg_p(_jitc->function->self.argi)) {
+	if (!(_jitc->function->self.call & jit_call_varargs))
+	    offset = 8 + _jitc->function->self.argf++;
+	else
+	    offset = _jitc->function->self.argi;
+	++_jitc->function->self.argi;
+    }
+   else {
 	offset = _jitc->function->self.size;
 	_jitc->function->self.size += sizeof(jit_word_t);
     }
-    return (jit_new_node_w(jit_code_arg_f, offset));
+    node = jit_new_node_ww(jit_code_arg_f, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 jit_node_t *
 _jit_arg_d(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
-
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function);
-    if (jit_arg_reg_p(_jitc->function->self.argi))
-	offset = _jitc->function->self.argi++;
-    else {
-	offset = _jitc->function->self.size;
-	_jitc->function->self.size += sizeof(jit_word_t);
+    if (jit_arg_reg_p(_jitc->function->self.argi)) {
+	if (!(_jitc->function->self.call & jit_call_varargs))
+	    offset = 8 + _jitc->function->self.argf++;
+	else
+	    offset = _jitc->function->self.argi;
+	++_jitc->function->self.argi;
     }
-    return (jit_new_node_w(jit_code_arg_d, offset));
+   else {
+	offset = _jitc->function->self.size;
+	_jitc->function->self.size += sizeof(jit_float64_t);
+    }
+    node = jit_new_node_ww(jit_code_arg_d, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 void
 _jit_getarg_c(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_c, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_c(u, _R32 + v->u.w);
     else
 	jit_ldxi_c(u, JIT_FP, v->u.w + C_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_uc(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_uc, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_uc(u, _R32 + v->u.w);
     else
 	jit_ldxi_uc(u, JIT_FP, v->u.w + C_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_s(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_s, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_s(u, _R32 + v->u.w);
     else
 	jit_ldxi_s(u, JIT_FP, v->u.w + S_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_us(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_us, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_us(u, _R32 + v->u.w);
     else
 	jit_ldxi_us(u, JIT_FP, v->u.w + S_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_i(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_i, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_i(u, _R32 + v->u.w);
     else
 	jit_ldxi_i(u, JIT_FP, v->u.w + I_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_ui(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_ui, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_ui(u, _R32 + v->u.w);
     else
 	jit_ldxi_ui(u, JIT_FP, v->u.w + I_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_l(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_l, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movr(u, _R32 + v->u.w);
     else
 	jit_ldxi(u, JIT_FP, v->u.w);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(putargr, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movr(_R32 + v->u.w, u);
     else
 	jit_stxi(v->u.w, JIT_FP, u);
+    jit_dec_synth();
 }
 
 void
@@ -500,6 +597,7 @@ _jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(putargi, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movi(_R32 + v->u.w, u);
     else {
@@ -508,26 +606,35 @@ _jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v)
 	jit_stxi(v->u.w, JIT_FP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_f(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_wp(getarg_f, u, v);
     if (jit_arg_reg_p(v->u.w))
-	jit_movr_f(u, _F8 + v->u.w);
+	jit_movr_w_f(u, _OUT0 + v->u.w);
+    else if (jit_arg_reg_p(v->u.w - 8))
+	jit_movr_f(u, _F8 + (v->u.w - 8));
     else
 	jit_ldxi_f(u, JIT_FP, v->u.w + F_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr_f(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_wp(putargr_f, u, v);
     if (jit_arg_reg_p(v->u.w))
-	jit_movr_f(_F8 + v->u.w, u);
+	jit_movr_f_w(_OUT0 + v->u.w, u);
+    else if (jit_arg_reg_p(v->u.w - 8))
+	jit_movr_f(_F8 + (v->u.w - 8), u);
     else
 	jit_stxi_f(v->u.w, JIT_FP, u + F_DISP);
+    jit_dec_synth();
 }
 
 void
@@ -535,34 +642,46 @@ _jit_putargi_f(jit_state_t *_jit, jit_float32_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_fp(putargi_f, u, v);
     if (jit_arg_reg_p(v->u.w))
-	jit_movi_f(_F8 + v->u.w, u);
+	jit_movi_f_w(_OUT0 + v->u.w, u);
+    else if (jit_arg_reg_p(v->u.w - 8))
+	jit_movi_f(_F8 + (v->u.w - 8), u);
     else {
 	regno = jit_get_reg(jit_class_fpr);
 	jit_movi_f(regno, u);
 	jit_stxi_f(v->u.w, JIT_FP, regno + F_DISP);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_d(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_wp(getarg_d, u, v);
     if (jit_arg_reg_p(v->u.w))
-	jit_movr_d(u, _F8 + v->u.w);
+	jit_movr_w_d(u, _OUT0 + v->u.w);
+    else if (jit_arg_reg_p(v->u.w - 8))
+	jit_movr_d(u, _F8 + (v->u.w - 8));
     else
 	jit_ldxi_d(u, JIT_FP, v->u.w);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr_d(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_wp(putargr_d, u, v);
     if (jit_arg_reg_p(v->u.w))
-	jit_movr_d(_F8 + v->u.w, u);
+	jit_movr_d_w(_OUT0 + v->u.w, u);
+    else if (jit_arg_reg_p(v->u.w - 8))
+	jit_movr_d(_F8 + (v->u.w - 8), u);
     else
 	jit_stxi_d(v->u.w, JIT_FP, u);
+    jit_dec_synth();
 }
 
 void
@@ -570,20 +689,26 @@ _jit_putargi_d(jit_state_t *_jit, jit_float64_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_dp(putargi_d, u, v);
     if (jit_arg_reg_p(v->u.w))
-	jit_movi_d(_F8 + v->u.w, u);
+	jit_movi_d_w(_OUT0 + v->u.w, u);
+    else if (jit_arg_reg_p(v->u.w - 8))
+	jit_movi_d(_F8 + (v->u.w - 8), u);
     else {
 	regno = jit_get_reg(jit_class_fpr);
 	jit_movi_d(regno, u);
 	jit_stxi_d(v->u.w, JIT_FP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function);
+    jit_inc_synth_w(pushargr, u);
+    jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movr(_OUT0 + _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -592,6 +717,7 @@ _jit_pushargr(jit_state_t *_jit, jit_int32_t u)
 	jit_stxi(_jitc->function->call.size + params_offset, JIT_SP, u);
 	_jitc->function->call.size += sizeof(jit_word_t);
     }
+    jit_dec_synth();
 }
 
 void
@@ -599,6 +725,8 @@ _jit_pushargi(jit_state_t *_jit, jit_word_t u)
 {
     jit_int32_t		 regno;
     assert(_jitc->function);
+    jit_inc_synth_w(pushargi, u);
+    jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movi(_OUT0 + _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -610,15 +738,20 @@ _jit_pushargi(jit_state_t *_jit, jit_word_t u)
 	_jitc->function->call.size += sizeof(jit_word_t);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function);
+    jit_inc_synth_w(pushargr_f, u);
+    jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
-	if (!(_jitc->function->call.call & jit_call_varargs))
-	    jit_movr_f(_F8 + _jitc->function->call.argi, u);
+	if (!(_jitc->function->call.call & jit_call_varargs)) {
+	    jit_movr_f(_F8 + _jitc->function->call.argf, u);
+	    ++_jitc->function->call.argf;
+	}
 	else
 	    jit_movr_f_w(_OUT0 + _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -628,6 +761,7 @@ _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
 		   JIT_SP, u);
 	_jitc->function->call.size += sizeof(jit_word_t);
     }
+    jit_dec_synth();
 }
 
 void
@@ -635,9 +769,13 @@ _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
 {
     jit_int32_t		 regno;
     assert(_jitc->function);
+    jit_inc_synth_f(pushargi_f, u);
+    jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
-	if (!(_jitc->function->call.call & jit_call_varargs))
-	    jit_movi_f(_F8 + _jitc->function->call.argi, u);
+	if (!(_jitc->function->call.call & jit_call_varargs)) {
+	    jit_movi_f(_F8 + _jitc->function->call.argf, u);
+	    ++_jitc->function->call.argf;
+	}
 	else
 	    jit_movi_f_w(_OUT0 + _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -650,23 +788,29 @@ _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
 	_jitc->function->call.size += sizeof(jit_word_t);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function);
+    jit_inc_synth_w(pushargr_d, u);
+    jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
-	if (!(_jitc->function->call.call & jit_call_varargs))
-	    jit_movr_d(_F8 + _jitc->function->call.argi, u);
+	if (!(_jitc->function->call.call & jit_call_varargs)) {
+	    jit_movr_d(_F8 + _jitc->function->call.argf, u);
+	    ++_jitc->function->call.argf;
+	}
 	else
 	    jit_movr_d_w(_OUT0 + _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
     }
     else {
 	jit_stxi_d(_jitc->function->call.size + params_offset, JIT_SP, u);
-	_jitc->function->call.size += sizeof(jit_word_t);
+	_jitc->function->call.size += sizeof(jit_float64_t);
     }
+    jit_dec_synth();
 }
 
 void
@@ -674,9 +818,13 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
 {
     jit_int32_t		 regno;
     assert(_jitc->function);
+    jit_inc_synth_d(pushargi_d, u);
+    jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
-	if (!(_jitc->function->call.call & jit_call_varargs))
-	    jit_movi_d(_F8 + _jitc->function->call.argi, u);
+	if (!(_jitc->function->call.call & jit_call_varargs)) {
+	    jit_movi_d(_F8 + _jitc->function->call.argf, u);
+	    ++_jitc->function->call.argf;
+	}
 	else
 	    jit_movi_d_w(_OUT0 + _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -685,9 +833,10 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
 	regno = jit_get_reg(jit_class_fpr);
 	jit_movi_d(regno, u);
 	jit_stxi_d(_jitc->function->call.size + params_offset, JIT_SP, regno);
-	_jitc->function->call.size += sizeof(jit_word_t);
+	_jitc->function->call.size += sizeof(jit_float64_t);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 jit_bool_t
@@ -702,6 +851,7 @@ _jit_finishr(jit_state_t *_jit, jit_int32_t r0)
 {
     jit_node_t		*call;
     assert(_jitc->function);
+    jit_inc_synth_w(finishr, r0);
     if (_jitc->function->self.alen < _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
     call = jit_callr(r0);
@@ -710,6 +860,7 @@ _jit_finishr(jit_state_t *_jit, jit_int32_t r0)
     _jitc->function->call.argi = _jitc->function->call.argf =
 	_jitc->function->call.size = 0;
     _jitc->prepare = 0;
+    jit_dec_synth();
 }
 
 jit_node_t *
@@ -717,6 +868,7 @@ _jit_finishi(jit_state_t *_jit, jit_pointer_t i0)
 {
     jit_node_t		*node;
     assert(_jitc->function);
+    jit_inc_synth_w(finishi, (jit_word_t)i0);
     if (_jitc->function->self.alen < _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
     node = jit_calli(i0);
@@ -725,61 +877,80 @@ _jit_finishi(jit_state_t *_jit, jit_pointer_t i0)
     _jitc->function->call.argi = _jitc->function->call.argf =
 	_jitc->function->call.size = 0;
     _jitc->prepare = 0;
+    jit_dec_synth();
     return (node);
 }
 
 void
 _jit_retval_c(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_c, r0);
     jit_extr_c(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_uc(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_uc, r0);
     jit_extr_uc(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_s(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_s, r0);
     jit_extr_s(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_us(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_us, r0);
     jit_extr_us(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_i(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_i, r0);
     jit_extr_i(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_ui(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_ui, r0);
     jit_extr_ui(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_l(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_l, r0);
     jit_movr(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_f(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_l, r0);
     jit_movr_f(r0, JIT_FRET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_d(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth_w(retval_d, r0);
     jit_movr_d(r0, JIT_FRET);
+    jit_dec_synth();
 }
 
 jit_pointer_t
@@ -793,9 +964,15 @@ _emit_code(jit_state_t *_jit)
     struct {
 	jit_node_t	*node;
 	jit_word_t	 word;
+#if DEVEL_DISASSEMBLER
+	jit_word_t	 prevw;
+#endif
 	jit_int32_t	 patch_offset;
 	jit_word_t	 prolog_offset;
     } undo;
+#if DEVEL_DISASSEMBLER
+    jit_word_t		 prevw;
+#endif
 
     _jitc->function = NULL;
 
@@ -810,6 +987,9 @@ _emit_code(jit_state_t *_jit)
     undo.node = NULL;
     undo.patch_offset = 0;
 
+#if DEVEL_DISASSEMBLER
+    prevw = _jit->pc.w;
+#endif
     undo.prolog_offset = 0;
     for (node = _jitc->head; node; node = node->next)
 	if (node->code != jit_code_label &&
@@ -915,8 +1095,15 @@ _emit_code(jit_state_t *_jit)
 	sync();
 #endif
 #if DEVEL_DISASSEMBLER
+	/* FIXME DEVEL_DISASSEMBLER should become DISASSEMBLER,
+	 * but a "real" DEVEL_DISASSEMBLER should be required
+	 * to turn the below "#if 0" into "#if 1" */
+#  if 0		/* Since disassembly outputs 3 instructions at a time,
+		 * make it "#if 1" for more clear debug output. */
 	sync();
-	node->offset = _jit->pc.w;
+#  endif
+	node->offset = (jit_uword_t)_jit->pc.w - (jit_uword_t)prevw;
+	prevw = _jit->pc.w;
 #endif
 	jit_regarg_set(node, value);
 	switch (node->code) {
@@ -1323,6 +1510,9 @@ _emit_code(jit_state_t *_jit)
 		_jitc->function = _jitc->functions.ptr + node->w.w;
 		undo.node = node;
 		undo.word = _jit->pc.w;
+#if DEVEL_DISASSEMBLER
+		undo.prevw = prevw;
+#endif
 		undo.patch_offset = _jitc->patches.offset;
 		undo.prolog_offset = _jitc->prolog.offset;
 	    restart_function:
@@ -1358,6 +1548,9 @@ _emit_code(jit_state_t *_jit)
 		    temp->flag &= ~jit_flag_patch;
 		    node = undo.node;
 		    _jit->pc.w = undo.word;
+#if DEVEL_DISASSEMBLER
+		    prevw = undo.prevw;
+#endif
 		    _jitc->patches.offset = undo.patch_offset;
 		    _jitc->prolog.offset = undo.prolog_offset;
 		    _jitc->ioff = 0;
@@ -1374,21 +1567,60 @@ _emit_code(jit_state_t *_jit)
 		_jitc->pred = 0;
 		_jitc->function = NULL;
 		break;
+	    case jit_code_va_start:
+		vastart(rn(node->u.w));
+		break;
+	    case jit_code_va_arg:
+		vaarg(rn(node->u.w), rn(node->v.w));
+		break;
+	    case jit_code_va_arg_d:
+		vaarg_d(rn(node->u.w), rn(node->v.w));
+		break;
 	    case jit_code_live:
-	    case jit_code_arg:
+	    case jit_code_arg:			case jit_code_ellipsis:
+	    case jit_code_allocai:		case jit_code_allocar:
 	    case jit_code_arg_f:		case jit_code_arg_d:
+	    case jit_code_va_end:
+	    case jit_code_ret:
+	    case jit_code_retr:			case jit_code_reti:
+	    case jit_code_retr_f:		case jit_code_reti_f:
+	    case jit_code_retr_d:		case jit_code_reti_d:
+	    case jit_code_getarg_c:		case jit_code_getarg_uc:
+	    case jit_code_getarg_s:		case jit_code_getarg_us:
+	    case jit_code_getarg_i:		case jit_code_getarg_ui:
+	    case jit_code_getarg_l:
+	    case jit_code_getarg_f:		case jit_code_getarg_d:
+	    case jit_code_putargr:		case jit_code_putargi:
+	    case jit_code_putargr_f:		case jit_code_putargi_f:
+	    case jit_code_putargr_d:		case jit_code_putargi_d:
+	    case jit_code_pushargr:		case jit_code_pushargi:
+	    case jit_code_pushargr_f:		case jit_code_pushargi_f:
+	    case jit_code_pushargr_d:		case jit_code_pushargi_d:
+	    case jit_code_retval_c:		case jit_code_retval_uc:
+	    case jit_code_retval_s:		case jit_code_retval_us:
+	    case jit_code_retval_i:
+	    case jit_code_retval_ui:		case jit_code_retval_l:
+	    case jit_code_retval_f:		case jit_code_retval_d:
+	    case jit_code_prepare:
+	    case jit_code_finishr:		case jit_code_finishi:
+		break;
+	    case jit_code_movr_w_f:
+		movr_w_f(rn(node->u.w), rn(node->v.w));
 		break;
 	    case jit_code_movr_f_w:
 		movr_f_w(rn(node->u.w), rn(node->v.w));
 		break;
+	    case jit_code_movr_w_d:
+		movr_w_d(rn(node->u.w), rn(node->v.w));
+		break;
 	    case jit_code_movr_d_w:
 		movr_d_w(rn(node->u.w), rn(node->v.w));
 		break;
 	    case jit_code_movi_f_w:
-		movi_f_w(rn(node->u.w), node->v.n->u.w);
+		movi_f_w(rn(node->u.w), node->v.n->u.p);
 		break;
 	    case jit_code_movi_d_w:
-		movi_d_w(rn(node->u.w), node->v.n->u.w);
+		movi_d_w(rn(node->u.w), node->v.n->u.p);
 		break;
 	    default:
 		abort();
@@ -1417,6 +1649,7 @@ _emit_code(jit_state_t *_jit)
 	    assert(jit_regset_scan1(&_jitc->regarg, 0) == jit_carry);
 	    assert(jit_regset_scan1(&_jitc->regarg, jit_carry + 1) == ULONG_MAX);
 	}
+	assert(_jitc->synth == 0);
 	/* update register live state */
 	jit_reglive(node);
     }
@@ -1459,7 +1692,14 @@ jit_flush(void *fptr, void *tptr)
     s = sysconf(_SC_PAGE_SIZE);
     f = (jit_word_t)fptr & -s;
     t = (((jit_word_t)tptr) + s - 1) & -s;
+#  if 0
     __clear_cache((void *)f, (void *)t);
+#  else
+    /* __clear_cache is a noop in (old?) gcc, but cache flush is
+     * required on a multi processor Linux system. */
+    for (s = f; s < t; s += 32)
+	asm volatile("fc %0" :: "r"(s) : "memory");
+#  endif
 #endif
 }
 
