@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012, 2013  Free Software Foundation, Inc.
+ * Copyright (C) 2012-2017  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -17,8 +17,6 @@
  *	Paulo Cesar Pereira de Andrade
  */
 
-#define rc(value)			jit_class_##value
-#define rn(reg)				(jit_regno(_rvs[jit_regno(reg)].spec))
 #define jit_arg_reg_p(i)		((i) >= 0 && (i) < 8)
 #define jit_arg_f_reg_p(i)		((i) >= 0 && (i) < 13)
 #if __BYTE_ORDER == __LITTLE_ENDIAN
@@ -33,6 +31,11 @@
 #  define F_DISP			(__WORDSIZE >> 3) - sizeof(jit_float32_t)
 #endif
 
+/*
+ * Types
+ */
+typedef jit_pointer_t jit_va_list_t;
+
 /*
  * Prototypes
  */
@@ -165,6 +168,10 @@ _jit_prolog(jit_state_t *_jit)
     jit_alloc((jit_pointer_t *)&_jitc->function->regoff,
 	      _jitc->reglen * sizeof(jit_int32_t));
 
+    /* _no_link here does not mean the jit_link() call can be removed
+     * by rewriting as:
+     * _jitc->function->prolog = jit_new_node(jit_code_prolog);
+     */
     _jitc->function->prolog = jit_new_node_no_link(jit_code_prolog);
     jit_link(_jitc->function->prolog);
     _jitc->function->prolog->w.w = offset;
@@ -189,73 +196,113 @@ _jit_allocai(jit_state_t *_jit, jit_int32_t length)
 	default:	_jitc->function->self.aoff &= -8;	break;
     }
     _jitc->function->self.aoff -= length;
+    if (!_jitc->realize) {
+	jit_inc_synth_ww(allocai, _jitc->function->self.aoff, length);
+	jit_dec_synth();
+    }
     return (_jitc->function->self.aoff);
 }
 
+void
+_jit_allocar(jit_state_t *_jit, jit_int32_t u, jit_int32_t v)
+{
+    jit_int32_t		 r0, r1;
+    assert(_jitc->function);
+    jit_inc_synth_ww(allocar, u, v);
+    if (!_jitc->function->allocar) {
+	_jitc->function->aoffoff = jit_allocai(sizeof(jit_int32_t));
+	_jitc->function->allocar = 1;
+    }
+    r0 = jit_get_reg(jit_class_gpr);
+    r1 = jit_get_reg(jit_class_gpr);
+    jit_ldr(r0, JIT_SP);
+    jit_negr(r1, v);
+    jit_andi(r1, r1, -16);
+    jit_ldxi_i(u, JIT_FP, _jitc->function->aoffoff);
+    jit_addr(u, u, r1);
+    jit_addr(JIT_SP, JIT_SP, r1);
+    jit_stxi_i(_jitc->function->aoffoff, JIT_FP, u);
+    jit_str(JIT_SP, r0);
+    jit_unget_reg(r1);
+    jit_unget_reg(r0);
+    jit_dec_synth();
+}
+
 void
 _jit_ret(jit_state_t *_jit)
 {
     jit_node_t		*instr;
-
     assert(_jitc->function);
-
+    jit_inc_synth(ret);
     /* jump to epilog */
     instr = jit_jmpi();
     jit_patch_at(instr, _jitc->function->epilog);
+    jit_dec_synth();
 }
 
 void
 _jit_retr(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr, u);
     if (JIT_RET != u)
 	jit_movr(JIT_RET, u);
     else
 	jit_live(JIT_RET);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti(jit_state_t *_jit, jit_word_t u)
 {
+    jit_inc_synth_w(reti, u);
     jit_movi(JIT_RET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_retr_f(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr_f, u);
     if (JIT_RET != u)
 	jit_movr_f(JIT_FRET, u);
     else
 	jit_live(JIT_FRET);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti_f(jit_state_t *_jit, jit_float32_t u)
 {
+    jit_inc_synth_f(reti_f, u);
     jit_movi_f(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_retr_d(jit_state_t *_jit, jit_int32_t u)
 {
+    jit_inc_synth_w(retr_d, u);
     if (JIT_FRET != u)
 	jit_movr_d(JIT_FRET, u);
     else
 	jit_live(JIT_FRET);
     jit_ret();
+    jit_dec_synth();
 }
 
 void
 _jit_reti_d(jit_state_t *_jit, jit_float64_t u)
 {
+    jit_inc_synth_d(reti_d, u);
     jit_movi_d(JIT_FRET, u);
     jit_ret();
+    jit_dec_synth();
 }
 
-/* must be called internally only */
 void
 _jit_epilog(jit_state_t *_jit)
 {
@@ -274,23 +321,48 @@ _jit_arg_register_p(jit_state_t *_jit, jit_node_t *u)
     return (jit_arg_f_reg_p(u->u.w));
 }
 
+void
+_jit_ellipsis(jit_state_t *_jit)
+{
+    jit_inc_synth(ellipsis);
+    if (_jitc->prepare) {
+	jit_link_prepare();
+	assert(!(_jitc->function->call.call & jit_call_varargs));
+	_jitc->function->call.call |= jit_call_varargs;
+    }
+    else {
+	jit_link_prolog();
+	assert(!(_jitc->function->self.call & jit_call_varargs));
+	_jitc->function->self.call |= jit_call_varargs;
+
+	_jitc->function->vagp = _jitc->function->self.argi;
+	_jitc->function->vafp = _jitc->function->self.argf;
+    }
+    jit_dec_synth();
+}
+
 jit_node_t *
 _jit_arg(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function);
     if (jit_arg_reg_p(_jitc->function->self.argi))
 	offset = _jitc->function->self.argi++;
     else
 	offset = _jitc->function->self.size;
     _jitc->function->self.size += sizeof(jit_word_t);
-    return (jit_new_node_w(jit_code_arg, offset));
+    node = jit_new_node_ww(jit_code_arg, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 jit_node_t *
 _jit_arg_f(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function);
     if (jit_arg_f_reg_p(_jitc->function->self.argf))
 	offset = _jitc->function->self.argf++;
@@ -304,13 +376,17 @@ _jit_arg_f(jit_state_t *_jit)
 #endif
     }
     _jitc->function->self.size += sizeof(jit_word_t);
-    return (jit_new_node_w(jit_code_arg_f, offset));
+    node = jit_new_node_ww(jit_code_arg_f, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 jit_node_t *
 _jit_arg_d(jit_state_t *_jit)
 {
-    jit_int32_t		offset;
+    jit_node_t		*node;
+    jit_int32_t		 offset;
     assert(_jitc->function);
     if (jit_arg_f_reg_p(_jitc->function->self.argf))
 	offset = _jitc->function->self.argf++;
@@ -324,53 +400,65 @@ _jit_arg_d(jit_state_t *_jit)
 #endif
     }
     _jitc->function->self.size += sizeof(jit_float64_t);
-    return (jit_new_node_w(jit_code_arg_d, offset));
+    node = jit_new_node_ww(jit_code_arg_d, offset,
+			   ++_jitc->function->self.argn);
+    jit_link_prolog();
+    return (node);
 }
 
 void
 _jit_getarg_c(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_c, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_c(u, JIT_RA0 - v->u.w);
     else
 	jit_ldxi_c(u, JIT_FP, v->u.w + C_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_uc(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_uc, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_uc(u, JIT_RA0 - v->u.w);
     else
 	jit_ldxi_uc(u, JIT_FP, v->u.w + C_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_s(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_s, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_s(u, JIT_RA0 - v->u.w);
     else
 	jit_ldxi_s(u, JIT_FP, v->u.w + S_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_us(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_us, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_us(u, JIT_RA0 - v->u.w);
     else
 	jit_ldxi_us(u, JIT_FP, v->u.w + S_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_i(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_i, u, v);
     if (jit_arg_reg_p(v->u.w)) {
 #if __WORDSIZE == 32
 	jit_movr(u, JIT_RA0 - v->u.w);
@@ -380,6 +468,7 @@ _jit_getarg_i(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
     }
     else
 	jit_ldxi_i(u, JIT_FP, v->u.w + I_DISP);
+    jit_dec_synth();
 }
 
 #if __WORDSIZE == 64
@@ -387,20 +476,24 @@ void
 _jit_getarg_ui(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_ui, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_extr_ui(u, JIT_RA0 - v->u.w);
     else
 	jit_ldxi_ui(u, JIT_FP, v->u.w + I_DISP);
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_l(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(getarg_l, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movr(u, JIT_RA0 - v->u.w);
     else
 	jit_ldxi_l(u, JIT_FP, v->u.w);
+    jit_dec_synth();
 }
 #endif
 
@@ -408,16 +501,19 @@ void
 _jit_putargr(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg);
+    jit_inc_synth_wp(putargr, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movr(JIT_RA0 - v->u.w, u);
     else
 	jit_stxi(v->u.w, JIT_FP, u);
+    jit_dec_synth();
 }
 
 void
 _jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
+    jit_inc_synth_wp(putargi, u, v);
     assert(v->code == jit_code_arg);
     if (jit_arg_reg_p(v->u.w))
 	jit_movi(JIT_RA0 - v->u.w, u);
@@ -427,26 +523,31 @@ _jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v)
 	jit_stxi(v->u.w, JIT_FP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_f(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_wp(getarg_f, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_d(u, JIT_FA0 - v->u.w);
     else
 	jit_ldxi_f(u, JIT_FP, v->u.w);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr_f(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_wp(putargr_f, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_d(JIT_FA0 - v->u.w, u);
     else
 	jit_stxi_f(v->u.w, JIT_FP, u);
+    jit_dec_synth();
 }
 
 void
@@ -454,6 +555,7 @@ _jit_putargi_f(jit_state_t *_jit, jit_float32_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg_f);
+    jit_inc_synth_fp(putargi_f, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movi_d(JIT_FA0 - v->u.w, u);
     else {
@@ -462,26 +564,31 @@ _jit_putargi_f(jit_state_t *_jit, jit_float32_t u, jit_node_t *v)
 	jit_stxi_f(v->u.w, JIT_FP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_getarg_d(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_wp(getarg_d, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_d(u, JIT_FA0 - v->u.w);
     else
 	jit_ldxi_d(u, JIT_FP, v->u.w);
+    jit_dec_synth();
 }
 
 void
 _jit_putargr_d(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_wp(putargr_d, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movr_d(JIT_FA0 - v->u.w, u);
     else
 	jit_stxi_d(v->u.w, JIT_FP, u);
+    jit_dec_synth();
 }
 
 void
@@ -489,6 +596,7 @@ _jit_putargi_d(jit_state_t *_jit, jit_float64_t u, jit_node_t *v)
 {
     jit_int32_t		regno;
     assert(v->code == jit_code_arg_d);
+    jit_inc_synth_dp(putargi_d, u, v);
     if (jit_arg_f_reg_p(v->u.w))
 	jit_movi_d(JIT_FA0 - v->u.w, u);
     else {
@@ -497,12 +605,15 @@ _jit_putargi_d(jit_state_t *_jit, jit_float64_t u, jit_node_t *v)
 	jit_stxi_d(v->u.w, JIT_FP, regno);
 	jit_unget_reg(regno);
     }
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function);
+    jit_inc_synth_w(pushargr, u);
+    jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movr(JIT_RA0 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -510,6 +621,7 @@ _jit_pushargr(jit_state_t *_jit, jit_int32_t u)
     else
 	jit_stxi(_jitc->function->call.size + params_offset, JIT_SP, u);
     _jitc->function->call.size += sizeof(jit_word_t);
+    jit_dec_synth();
 }
 
 void
@@ -517,6 +629,8 @@ _jit_pushargi(jit_state_t *_jit, jit_word_t u)
 {
     jit_int32_t		 regno;
     assert(_jitc->function);
+    jit_inc_synth_w(pushargi, u);
+    jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
 	jit_movi(JIT_RA0 - _jitc->function->call.argi, u);
 	++_jitc->function->call.argi;
@@ -528,29 +642,35 @@ _jit_pushargi(jit_state_t *_jit, jit_word_t u)
 	jit_unget_reg(regno);
     }
     _jitc->function->call.size += sizeof(jit_word_t);
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function);
-    if (jit_arg_f_reg_p(_jitc->function->call.argf)) {
+    jit_inc_synth_w(pushargr_f, u);
+    jit_link_prepare();
+    if (jit_arg_f_reg_p(_jitc->function->call.argf) &&
+	!(_jitc->function->call.call & jit_call_varargs)) {
 	jit_movr_d(JIT_FA0 - _jitc->function->call.argf, u);
 	++_jitc->function->call.argf;
-	if (!(_jitc->function->call.call & jit_call_varargs)) {
-	    /* in case of excess arguments */
-	    if (jit_arg_reg_p(_jitc->function->call.argi)) {
+	/* in case of excess arguments */
+	if (jit_arg_reg_p(_jitc->function->call.argi)) {
 #if __WORDSIZE == 32
-		_jitc->function->call.argi += 2;
+	    _jitc->function->call.argi += 2;
+	    if (!jit_arg_reg_p(_jitc->function->call.argi - 1))
+		--_jitc->function->call.argi;
 #else
-		_jitc->function->call.argi++;
+	    _jitc->function->call.argi++;
 #endif
-	    }
-	    _jitc->function->call.size += sizeof(jit_word_t);
-	    return;
 	}
     }
-    if (jit_arg_reg_p(_jitc->function->call.argi)) {
+    else if (jit_arg_reg_p(_jitc->function->call.argi
+#if __WORDSIZE == 32
+			  + 1
+#endif
+			   )) {
 	/* use reserved 8 bytes area */
 	jit_stxi_d(alloca_offset - 8, JIT_FP, u);
 	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
@@ -566,72 +686,81 @@ _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
 	jit_stxi_f(_jitc->function->call.size + params_offset + F_DISP,
 		   JIT_SP, u);
     _jitc->function->call.size += sizeof(jit_word_t);
+    jit_dec_synth();
 }
 
 void
 _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
 {
     jit_int32_t		 regno;
-
     assert(_jitc->function);
-    if (jit_arg_f_reg_p(_jitc->function->call.argf)) {
+    jit_inc_synth_f(pushargi_f, u);
+    jit_link_prepare();
+    if (jit_arg_f_reg_p(_jitc->function->call.argf) &&
+	!(_jitc->function->call.call & jit_call_varargs)) {
 	jit_movi_d(JIT_FA0 - _jitc->function->call.argf, u);
 	++_jitc->function->call.argf;
-	if (!(_jitc->function->call.call & jit_call_varargs)) {
 	    /* in case of excess arguments */
-	    if (jit_arg_reg_p(_jitc->function->call.argi)) {
 #if __WORDSIZE == 32
-		_jitc->function->call.argi += 2;
+	_jitc->function->call.argi += 2;
+	if (!jit_arg_reg_p(_jitc->function->call.argi - 1))
+	    --_jitc->function->call.argi;
 #else
-		_jitc->function->call.argi++;
-#endif
-	    }
-	    _jitc->function->call.size += sizeof(jit_word_t);
-	    return;
-	}
-    }
-    regno = jit_get_reg(jit_class_fpr);
-    jit_movi_f(regno, u);
-    if (jit_arg_reg_p(_jitc->function->call.argi)) {
-	/* use reserved 8 bytes area */
-	jit_stxi_d(alloca_offset - 8, JIT_FP, regno);
-	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		 alloca_offset - 8);
 	_jitc->function->call.argi++;
+#endif
+    }
+    else {
+	regno = jit_get_reg(jit_class_fpr);
+	jit_movi_f(regno, u);
+	if (jit_arg_reg_p(_jitc->function->call.argi
 #if __WORDSIZE == 32
-	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		 alloca_offset - 4);
-	_jitc->function->call.argi++;
+			  + 1
 #endif
+			  )) {
+	    /* use reserved 8 bytes area */
+	    jit_stxi_d(alloca_offset - 8, JIT_FP, regno);
+	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
+		     alloca_offset - 8);
+	    _jitc->function->call.argi++;
+#if __WORDSIZE == 32
+	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
+		     alloca_offset - 4);
+	    _jitc->function->call.argi++;
+#endif
+	}
+	else
+	    jit_stxi_f(_jitc->function->call.size + params_offset + F_DISP,
+		       JIT_SP, regno);
+	jit_unget_reg(regno);
     }
-    else
-	jit_stxi_f(_jitc->function->call.size + params_offset + F_DISP,
-		   JIT_SP, regno);
     _jitc->function->call.size += sizeof(jit_word_t);
-    jit_unget_reg(regno);
+    jit_dec_synth();
 }
 
 void
 _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
 {
     assert(_jitc->function);
-    if (jit_arg_f_reg_p(_jitc->function->call.argf)) {
+    jit_inc_synth_w(pushargr_d, u);
+    jit_link_prepare();
+    if (jit_arg_f_reg_p(_jitc->function->call.argf) &&
+	!(_jitc->function->call.call & jit_call_varargs)) {
 	jit_movr_d(JIT_FA0 - _jitc->function->call.argf, u);
 	++_jitc->function->call.argf;
-	if (!(_jitc->function->call.call & jit_call_varargs)) {
 	    /* in case of excess arguments */
-	    if (jit_arg_reg_p(_jitc->function->call.argi)) {
 #if __WORDSIZE == 32
-		_jitc->function->call.argi += 2;
+	_jitc->function->call.argi += 2;
+	if (!jit_arg_reg_p(_jitc->function->call.argi - 1))
+	    --_jitc->function->call.argi;
 #else
-		_jitc->function->call.argi++;
+	_jitc->function->call.argi++;
 #endif
-	    }
-	    _jitc->function->call.size += sizeof(jit_float64_t);
-	    return;
-	}
     }
-    if (jit_arg_reg_p(_jitc->function->call.argi)) {
+    else if (jit_arg_reg_p(_jitc->function->call.argi
+#if __WORDSIZE == 32
+			  + 1
+#endif
+			   )) {
 	/* use reserved 8 bytes area */
 	jit_stxi_d(alloca_offset - 8, JIT_FP, u);
 	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
@@ -643,51 +772,76 @@ _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
 	_jitc->function->call.argi++;
 #endif
     }
-    else
+    else {
 	jit_stxi_d(_jitc->function->call.size + params_offset, JIT_SP, u);
+#if __WORDSIZE == 32
+	    if (jit_arg_reg_p(_jitc->function->call.argi)) {
+		jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_SP,
+			 _jitc->function->call.size + params_offset);
+		_jitc->function->call.argi++;
+	    }
+#endif
+    }
     _jitc->function->call.size += sizeof(jit_float64_t);
+    jit_dec_synth();
 }
 
 void
 _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
 {
     jit_int32_t		 regno;
-
     assert(_jitc->function);
-    if (jit_arg_f_reg_p(_jitc->function->call.argf)) {
+    jit_inc_synth_d(pushargi_d, u);
+    jit_link_prepare();
+    if (jit_arg_f_reg_p(_jitc->function->call.argf) &&
+	!(_jitc->function->call.call & jit_call_varargs)) {
 	jit_movi_d(JIT_FA0 - _jitc->function->call.argf, u);
 	++_jitc->function->call.argf;
-	if (!(_jitc->function->call.call & jit_call_varargs)) {
-	    /* in case of excess arguments */
-	    if (jit_arg_reg_p(_jitc->function->call.argi)) {
+	/* in case of excess arguments */
+	if (jit_arg_reg_p(_jitc->function->call.argi)) {
 #if __WORDSIZE == 32
-		_jitc->function->call.argi += 2;
+	    _jitc->function->call.argi += 2;
+	    if (!jit_arg_reg_p(_jitc->function->call.argi - 1))
+		--_jitc->function->call.argi;
 #else
-		_jitc->function->call.argi++;
+	    _jitc->function->call.argi++;
 #endif
-	    }
-	    _jitc->function->call.size += sizeof(jit_float64_t);
-	    return;
 	}
     }
-    regno = jit_get_reg(jit_class_fpr);
-    jit_movi_d(regno, u);
-    if (jit_arg_reg_p(_jitc->function->call.argi)) {
-	/* use reserved 8 bytes area */
-	jit_stxi_d(alloca_offset - 8, JIT_FP, regno);
-	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		 alloca_offset - 8);
-	_jitc->function->call.argi++;
+    else {
+	regno = jit_get_reg(jit_class_fpr);
+	jit_movi_d(regno, u);
+	if (jit_arg_reg_p(_jitc->function->call.argi
 #if __WORDSIZE == 32
-	jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
-		 alloca_offset - 4);
-	_jitc->function->call.argi++;
+			  + 1
 #endif
+			  )) {
+	    /* use reserved 8 bytes area */
+	    jit_stxi_d(alloca_offset - 8, JIT_FP, regno);
+	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
+		     alloca_offset - 8);
+	    _jitc->function->call.argi++;
+#if __WORDSIZE == 32
+	    jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_FP,
+		     alloca_offset - 4);
+	    _jitc->function->call.argi++;
+#endif
+	}
+	else {
+	    jit_stxi_d(_jitc->function->call.size + params_offset,
+		       JIT_SP, regno);
+#if __WORDSIZE == 32
+	    if (jit_arg_reg_p(_jitc->function->call.argi)) {
+		jit_ldxi(JIT_RA0 - _jitc->function->call.argi, JIT_SP,
+			 _jitc->function->call.size + params_offset);
+		_jitc->function->call.argi++;
+	    }
+#endif
+	}
+	jit_unget_reg(regno);
     }
-    else
-	jit_stxi_d(_jitc->function->call.size + params_offset, JIT_SP, regno);
     _jitc->function->call.size += sizeof(jit_float64_t);
-    jit_unget_reg(regno);
+    jit_dec_synth();
 }
 
 jit_bool_t
@@ -715,6 +869,7 @@ _jit_finishr(jit_state_t *_jit, jit_int32_t r0)
 {
     jit_node_t		*call;
     assert(_jitc->function);
+    jit_inc_synth_w(finishr, r0);
     if (_jitc->function->self.alen < _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
     call = jit_callr(r0);
@@ -722,6 +877,7 @@ _jit_finishr(jit_state_t *_jit, jit_int32_t r0)
     call->w.w = _jitc->function->call.argf;
     _jitc->function->call.argi = _jitc->function->call.argf = 0;
     _jitc->prepare = 0;
+    jit_dec_synth();
 }
 
 jit_node_t *
@@ -729,6 +885,7 @@ _jit_finishi(jit_state_t *_jit, jit_pointer_t i0)
 {
     jit_node_t		*node;
     assert(_jitc->function);
+    jit_inc_synth_w(finishi, (jit_word_t)i0);
     if (_jitc->function->self.alen < _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
     node = jit_calli(i0);
@@ -736,70 +893,89 @@ _jit_finishi(jit_state_t *_jit, jit_pointer_t i0)
     node->w.w = _jitc->function->call.argf;
     _jitc->function->call.argi = _jitc->function->call.argf = 0;
     _jitc->prepare = 0;
+    jit_dec_synth();
     return (node);
 }
 
 void
 _jit_retval_c(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth(retval_c);
     jit_extr_c(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_uc(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth(retval_uc);
     jit_extr_uc(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_s(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth(retval_s);
     jit_extr_s(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_us(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth(retval_us);
     jit_extr_us(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_i(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth(retval_i);
 #if __WORDSIZE == 32
     if (r0 != JIT_RET)
 	jit_movr(r0, JIT_RET);
 #else
     jit_extr_i(r0, JIT_RET);
 #endif
+    jit_dec_synth();
 }
 
 #if __WORDSIZE == 64
 void
 _jit_retval_ui(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth(retval_ui);
     jit_extr_ui(r0, JIT_RET);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_l(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth(retval_l);
     if (r0 != JIT_RET)
 	jit_movr(r0, JIT_RET);
+    jit_dec_synth();
 }
 #endif
 
 void
 _jit_retval_f(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth(retval_f);
     jit_retval_d(r0);
+    jit_dec_synth();
 }
 
 void
 _jit_retval_d(jit_state_t *_jit, jit_int32_t r0)
 {
+    jit_inc_synth(retval_d);
     if (r0 != JIT_FRET)
 	jit_movr_d(r0, JIT_FRET);
+    jit_dec_synth();
 }
 
 jit_pointer_t
@@ -813,11 +989,17 @@ _emit_code(jit_state_t *_jit)
     struct {
 	jit_node_t	*node;
 	jit_word_t	 word;
+#if DEVEL_DISASSEMBLER
+	jit_word_t	 prevw;
+#endif
 	jit_word_t	 patch_offset;
 #if __powerpc__
 	jit_word_t	 prolog_offset;
 #endif
     } undo;
+#if DEVEL_DISASSEMBLER
+    jit_word_t		 prevw;
+#endif
 
     _jitc->function = NULL;
 
@@ -827,6 +1009,9 @@ _emit_code(jit_state_t *_jit)
     undo.node = NULL;
     undo.patch_offset = 0;
 
+#if DEVEL_DISASSEMBLER
+    prevw = _jit->pc.w;
+#endif
 #if __powerpc__ && !ABI_ELFv2
     undo.prolog_offset = 0;
     for (node = _jitc->head; node; node = node->next)
@@ -932,7 +1117,8 @@ _emit_code(jit_state_t *_jit)
 	    return (NULL);
 
 #if DEVEL_DISASSEMBLER
-	node->offset = _jit->pc.w;
+	node->offset = (jit_uword_t)_jit->pc.w - (jit_uword_t)prevw;
+	prevw = _jit->pc.w;
 #endif
 	value = jit_classify(node->code);
 	jit_regarg_set(node, value);
@@ -1349,6 +1535,9 @@ _emit_code(jit_state_t *_jit)
 		_jitc->function = _jitc->functions.ptr + node->w.w;
 		undo.node = node;
 		undo.word = _jit->pc.w;
+#if DEVEL_DISASSEMBLER
+		undo.prevw = prevw;
+#endif
 		undo.patch_offset = _jitc->patches.offset;
 #if __powerpc__ && !ABI_ELFv2
 		undo.prolog_offset = _jitc->prolog.offset;
@@ -1389,6 +1578,9 @@ _emit_code(jit_state_t *_jit)
 		    temp->flag &= ~jit_flag_patch;
 		    node = undo.node;
 		    _jit->pc.w = undo.word;
+#if DEVEL_DISASSEMBLER
+		    prevw = undo.prevw;
+#endif
 		    _jitc->patches.offset = undo.patch_offset;
 #if __powerpc__ && !ABI_ELFv2
 		    _jitc->prolog.offset = undo.prolog_offset;
@@ -1401,15 +1593,52 @@ _emit_code(jit_state_t *_jit)
 		epilog(node);
 		_jitc->function = NULL;
 		break;
+	    case jit_code_va_start:
+		vastart(rn(node->u.w));
+		break;
+	    case jit_code_va_arg:
+		vaarg(rn(node->u.w), rn(node->v.w));
+		break;
+	    case jit_code_va_arg_d:
+		vaarg_d(rn(node->u.w), rn(node->v.w));
+		break;
 	    case jit_code_live:
-	    case jit_code_arg:
+	    case jit_code_arg:			case jit_code_ellipsis:
+	    case jit_code_allocai:		case jit_code_allocar:
 	    case jit_code_arg_f:		case jit_code_arg_d:
+	    case jit_code_va_end:
+	    case jit_code_ret:
+	    case jit_code_retr:			case jit_code_reti:
+	    case jit_code_retr_f:		case jit_code_reti_f:
+	    case jit_code_retr_d:		case jit_code_reti_d:
+	    case jit_code_getarg_c:		case jit_code_getarg_uc:
+	    case jit_code_getarg_s:		case jit_code_getarg_us:
+	    case jit_code_getarg_i:
+#if __WORDSIZE == 64
+	    case jit_code_getarg_ui:		case jit_code_getarg_l:
+#endif
+	    case jit_code_getarg_f:		case jit_code_getarg_d:
+	    case jit_code_putargr:		case jit_code_putargi:
+	    case jit_code_putargr_f:		case jit_code_putargi_f:
+	    case jit_code_putargr_d:		case jit_code_putargi_d:
+	    case jit_code_pushargr:		case jit_code_pushargi:
+	    case jit_code_pushargr_f:		case jit_code_pushargi_f:
+	    case jit_code_pushargr_d:		case jit_code_pushargi_d:
+	    case jit_code_retval_c:		case jit_code_retval_uc:
+	    case jit_code_retval_s:		case jit_code_retval_us:
+	    case jit_code_retval_i:
+#if __WORDSIZE == 64
+	    case jit_code_retval_ui:		case jit_code_retval_l:
+#endif
+	    case jit_code_retval_f:		case jit_code_retval_d:
+	    case jit_code_prepare:
+	    case jit_code_finishr:		case jit_code_finishi:
 		break;
 	    default:
 		abort();
 	}
 	jit_regarg_clr(node, value);
-	assert(_jitc->regarg == 0);
+	assert(_jitc->regarg == 0 && _jitc->synth == 0);
 	/* update register live state */
 	jit_reglive(node);
     }
