@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012, 2013  Free Software Foundation, Inc.
+ * Copyright (C) 2012-2017  Free Software Foundation, Inc.
  *
  * This file is part of GNU lightning.
  *
@@ -34,7 +34,6 @@
 #  define _ST4_REGNO			4
 #  define _ST5_REGNO			5
 #  define _ST6_REGNO			6
-#  define _ST7_REGNO			7
 #  define x87rx(code, md, rb, ri, ms)	_x87rx(_jit, code, md, rb, ri, ms)
 #  define fldcwm(md, rb, ri, ms)	x87rx(015, md, rb, ri, ms)
 #  define fstcwm(md, rb, ri, ms)	_fstcwm(_jit, md, rb, ri, ms)
@@ -887,8 +886,16 @@ _x87_ldi_f(jit_state_t *_jit, jit_int32_t r0, jit_word_t i0)
 static void
 _x87_ldxr_f(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_int32_t r2)
 {
+#if __X64_32
+    jit_int32_t		reg;
+    reg = jit_get_reg(jit_class_gpr);
+    addr(rn(reg), r1, r2);
+    x87_ldr_f(r0, rn(reg));
+    jit_unget_reg(reg);
+#else
     fldsm(0, r1, r2, _SCL1);
     fstpr(r0 + 1);
+#endif
 }
 
 static void
@@ -901,8 +908,13 @@ _x87_ldxi_f(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_word_t i0)
     }
     else {
 	reg = jit_get_reg(jit_class_gpr);
+#if __X64_32
+	addi(rn(reg), r1, i0);
+	x87_ldr_f(r0, rn(reg));
+#else
 	movi(rn(reg), i0);
 	x87_ldxr_f(r0, r1, rn(reg));
+#endif
 	jit_unget_reg(reg);
     }
 }
@@ -941,6 +953,13 @@ _x87_sti_f(jit_state_t *_jit, jit_word_t i0, jit_int32_t r0)
 static void
 _x87_stxr_f(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_int32_t r2)
 {
+#if __X64_32
+    jit_int32_t		reg;
+    reg = jit_get_reg(jit_class_gpr);
+    addr(rn(reg), r0, r1);
+    x87_str_f(rn(reg), r2);
+    jit_unget_reg(reg);
+#else
     if (r2 == _ST0_REGNO)
 	fstsm(0, r0, r1, _SCL1);
     else {
@@ -948,6 +967,7 @@ _x87_stxr_f(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_int32_t r2)
 	fstsm(0, r0, r1, _SCL1);
 	fxchr(r2);
     }
+#endif
 }
 
 static void
@@ -956,8 +976,13 @@ _x87_stxi_f(jit_state_t *_jit, jit_word_t i0, jit_int32_t r0, jit_int32_t r1)
     jit_int32_t		reg;
     if (!can_sign_extend_int_p(i0)) {
 	reg = jit_get_reg(jit_class_gpr);
+#if __X64_32
+	addi(rn(reg), r0, i0);
+	x87_str_f(rn(reg), r1);
+#else
 	movi(rn(reg), i0);
 	x87_stxr_f(rn(reg), r0, r1);
+#endif
 	jit_unget_reg(reg);
     }
     else if (r1 == _ST0_REGNO)
@@ -1144,8 +1169,16 @@ _x87_ldi_d(jit_state_t *_jit, jit_int32_t r0, jit_word_t i0)
 static void
 _x87_ldxr_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_int32_t r2)
 {
+#if __X64_32
+    jit_int32_t		reg;
+    reg = jit_get_reg(jit_class_gpr);
+    addr(rn(reg), r1, r2);
+    x87_ldr_d(r0, rn(reg));
+    jit_unget_reg(reg);
+#else
     fldlm(0, r1, r2, _SCL1);
     fstpr(r0 + 1);
+#endif
 }
 
 static void
@@ -1158,8 +1191,13 @@ _x87_ldxi_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_word_t i0)
     }
     else {
 	reg = jit_get_reg(jit_class_gpr);
+#if __X64_32
+	addi(rn(reg), r1, i0);
+	x87_ldr_d(r0, rn(reg));
+#else
 	movi(rn(reg), i0);
 	x87_ldxr_d(r0, r1, rn(reg));
+#endif
 	jit_unget_reg(reg);
     }
 }
@@ -1198,6 +1236,13 @@ _x87_sti_d(jit_state_t *_jit, jit_word_t i0, jit_int32_t r0)
 static void
 _x87_stxr_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_int32_t r2)
 {
+#if __X64_32
+    jit_int32_t		reg;
+    reg = jit_get_reg(jit_class_gpr);
+    addr(rn(reg), r0, r1);
+    x87_str_d(rn(reg), r2);
+    jit_unget_reg(reg);
+#else
     if (r2 == _ST0_REGNO)
 	fstlm(0, r0, r1, _SCL1);
     else {
@@ -1205,6 +1250,7 @@ _x87_stxr_d(jit_state_t *_jit, jit_int32_t r0, jit_int32_t r1, jit_int32_t r2)
 	fstlm(0, r0, r1, _SCL1);
 	fxchr(r2);
     }
+#endif
 }
 
 static void
@@ -1213,8 +1259,13 @@ _x87_stxi_d(jit_state_t *_jit, jit_word_t i0, jit_int32_t r0, jit_int32_t r1)
     jit_int32_t		reg;
     if (!can_sign_extend_int_p(i0)) {
 	reg = jit_get_reg(jit_class_gpr);
+#if __X64_32
+	addi(rn(reg), r0, i0);
+	x87_str_d(rn(reg), r1);
+#else
 	movi(rn(reg), i0);
 	x87_stxr_d(rn(reg), r0, r1);
+#endif
 	jit_unget_reg(reg);
     }
     else if (r1 == _ST0_REGNO)
